/*
 * cleardatascreen.c - Screen for rebooting or halting
 * Copyright (C) 2024  Alexander Rosenberg
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version. See the LICENSE file for more information.
 */
#include "cleardatascreen.h"

#include <err.h>

static bool clear_data_screen_display(ClearDataScreen *screen,
                                      SensorState *state) {
    if (state->back_down || (!screen->choice && state->sel_down) ||
        (screen->done && (state->sel_down || state->up_down ||
                          state->down_down))) {
        screen->done = false;
        screen->error = false;
        lcd_display_control(state->lcd, LCD_CURSOR_NO_BLINK, LCD_CURSOR_OFF,
                            LCD_DISPLAY_ON);
        return true;
    }
    if (state->sel_down) {
        // TODO clear data
        char *errmsg;
        sqlite3_exec(state->db, "DELETE FROM env_data;", NULL, NULL, &errmsg);
        if (errmsg) {
            warnx("sqlite3 error when clearing database: \"%s\"\n", errmsg);
            sqlite3_free(errmsg);
            screen->error = true;
        } else {
            LOG_VERBOSE("Database cleared!\n");
        }
        screen->need_redraw = true;
        screen->done = true;
        return false;
    }
    if (screen->need_redraw || state->force_draw || state->up_down ||
        state->down_down) {
        screen->need_redraw = false;
        screen->choice = abs((screen->choice + state->up_down -
                              state->down_down) % 2);
        lcd_clear(state->lcd);
        lcd_move_to(state->lcd, 0, 0);
        if (screen->done) {
            lcd_display_control(state->lcd, LCD_CURSOR_NO_BLINK, LCD_CURSOR_OFF,
                                LCD_DISPLAY_ON);
            lcd_write_string(state->lcd, screen->error ? "Error!" : "Done!");
        } else {
            lcd_write_string(state->lcd, "Really clear?");
            lcd_move_to(state->lcd, 1, 0);
            lcd_write_string(state->lcd, ">No >Yes");
            lcd_move_to(state->lcd, 1, 4 * screen->choice);
            lcd_display_control(state->lcd, LCD_CURSOR_BLINK, LCD_CURSOR_ON,
                                LCD_DISPLAY_ON);
        }
    }
    return false;
}

ClearDataScreen *clear_data_screen_new() {
    ClearDataScreen *s = malloc_checked(sizeof(ClearDataScreen));
    screen_init(&s->parent, "Clear data",
                (ScreenDispatchFunc) clear_data_screen_display,
                (ScreenCleanupFunc) free);
    s->choice = 0;
    s->need_redraw = false;
    s->done = false;
    return s;
}
